﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Telerik.WinControls.UI;

namespace _978950_RadGridViewPrintingHeaders
{
    static class DragDropHelper
    {
        public static void MoveOnLeftOrRight<T>(bool isDroppedAtLeft, Collection<T> collection, T targetItem, T draggedItem)
        {
            int targetIndex = collection.IndexOf(targetItem);
            int draggedItemIndex = collection.IndexOf(draggedItem);
            int itemsCount = collection.Count;

            CalculateTargetIndex(isDroppedAtLeft, itemsCount, ref targetIndex, ref draggedItemIndex);
            Move<T>(collection, draggedItemIndex, targetIndex);
        }

        public static void CalculateTargetIndex(bool isDroppedAtLeft, int itemsCount, ref int targetIndex, ref int draggedItemIndex)
        {
            if (isDroppedAtLeft)
            {
                int indexDifference = draggedItemIndex - targetIndex;

                if (indexDifference < 0)
                {
                    if (targetIndex > 0)
                    {
                        targetIndex--;
                    }
                }
            }
            else
            {
                int indexDifference = draggedItemIndex - targetIndex;

                if (indexDifference > 0)
                {
                    if (targetIndex + 1 < itemsCount)
                    {
                        targetIndex++;
                    }
                }

            }
        }

        public static void Move<T>(Collection<T> collection, int oldIndex, int newIndex)
        {
            Telerik.WinControls.Data.ObservableCollection<T> observableCollection = collection as Telerik.WinControls.Data.ObservableCollection<T>;
            if (observableCollection != null)
            {
                observableCollection.Move(oldIndex, newIndex);
            }
            else
            {
                T item = collection[oldIndex];
                collection.RemoveAt(oldIndex);
                collection.Insert(newIndex, item);
            }
        }

        public static RadPosition GetDropPosition(Point dropLocation, Size size)
        {
            RadPosition result = RadPosition.None;

            int halfWidth = size.Width / 2;
            int thirdWidth = size.Width / 3;
            int twoThirdsWidth = 2 * size.Width / 3;

            if (dropLocation.X <= halfWidth)
                result |= RadPosition.Left;
            else
                result |= RadPosition.Right;

            if (thirdWidth < dropLocation.X && dropLocation.X < twoThirdsWidth)
            {
                int halfHeight = size.Height / 2;

                if (dropLocation.Y <= halfHeight)
                    result |= RadPosition.Top;
                else
                    result |= RadPosition.Bottom;
            }

            return result;
        }

    }
}
